@@ -35,20 +35,21 @@
 #include <unistd.h>
 #include <sys/time.h>
 #include <sys/select.h>
-#include <string.h>
 #include <signal.h>
 #include <arpa/inet.h>
 #include <errno.h>
 
 #include <networking.h>
 
-#include <macros.h>
+#include <xmalloc.h>
+#include <xstring.h>
 #include <state.h>
 #include <strtotype.h>
 #include <logutils.h>
 #include <response.h>
 #include <serverconf.h>
 #include <reutils.h>
+#include <anssettings.h>
 
 /* This is here till i get the proper macro included */
 #undef max
@@ -71,7 +72,7 @@ set_authport(char* port) {
 
         if( authport != server.authport ) {
 		server.authport = authport;
-        	server.is_modifed.authport = TRUE;
+        	server.is_modifed.authport = true;
 	}
         return 0;
 } /* -- end set_authport() -- */
@@ -108,7 +109,7 @@ set_max_queries( char* max_queries ) {
 
 	if( q != server.max_queries ) {
 		server.max_queries = (uint) q;
-		server.is_modifed.max_queries = TRUE;
+		server.is_modifed.max_queries = true;
 	}
 
 	return 0;
@@ -163,8 +164,8 @@ free_connection_context( ConnContext* cc ) {
 	if(!cc)
 		return;
 	kill_connection(cc);
-	if_notnull_free(cc->answer)
-	free(cc);
+	xnullfree(cc->answer);
+	xfree(cc);
 } /* -- end free_connection_context() -- */
 
 /* shutdown and close a socket */
@@ -181,7 +182,6 @@ kill_connection(ConnContext* cc) {
 /* Initializes a ConnContext */
 void
 clear_connection_context(ConnContext* cc) {
-	memset(cc, 0, sizeof(ConnContext));
 	cc->next = NULL;
 	cc->answer = NULL;
 	cc->qsock = -1;
@@ -190,12 +190,8 @@ clear_connection_context(ConnContext* cc) {
 /* Creates and returns a cleared ConnContext */
 ConnContext*
 new_connection_context( void ) {
-	ConnContext* cc = malloc(sizeof(ConnContext));
-	if( ! cc )
-		choke("Could not create new ConnContext");
-	else
-		clear_connection_context(cc);
-
+	ConnContext* cc = xnew0(ConnContext, 1);
+	clear_connection_context(cc);
 	return cc;
 } /* -- end new_connection_context() -- */
 
@@ -327,7 +323,7 @@ read_query(ConnContext* cc) {
 	}
 	else {
 		dreport("%d bytes read; total bytes read == %d", bytes, bytes + cc->query_len);
-		if( strlen(buf) < bytes ) {
+		if( xstrlen(buf) < bytes ) {
 			dreport("Emedded '\\0'");
 			kill_connection(cc);
 			return;
@@ -338,7 +334,7 @@ read_query(ConnContext* cc) {
 			return;
 		}
 		strcat((char*) &(cc->query), buf);
-		cc->query_len = strlen(cc->query);
+		cc->query_len = xstrlen(cc->query);
 	}
 
 	/* Check for a CR or LF */
@@ -350,7 +346,7 @@ read_query(ConnContext* cc) {
 			kill_connection(cc);
 			return;
 		}
-		cc->finished_read = TRUE;
+		cc->finished_read = true;
 	}
 
 	return;
@@ -405,30 +401,26 @@ set_response(ConnContext* cc, char* answer) {
 		carp("Invalid ConnContext");
 		return -1;
 	}
-	if( is_nil(answer) ) {
+	if( xstr_is_nil(answer) ) {
 		carp("Invalid answer");
 		return -1;
 	}
 
 	dreport("Setting response");
 	cc->answer = answer;
-	char* buf = strprintf("%d , %d : %s\r\n",
+	char* buf = xsprintf("%d , %d : %s\r\n",
 			cc->lport, cc->rport, answer);
-	if( ! buf ) {
-		choke("Could not make room for response");
-		return -1;
-	}
 	/* Truncate to 512 bytes */
-	if(strlen(buf) > MSG_SIZE - 2) {
+	if(xstrlen(buf) > MSG_SIZE - 2) {
 		buf[MSG_SIZE - 4] = '\r';
 		buf[MSG_SIZE - 3] = '\n';
 		buf[MSG_SIZE - 2] = 0;
 		dreport("response truncated");
 	}
-	cc->response_len = strlen(buf);
+	cc->response_len = xstrlen(buf);
 	memcpy(&(cc->response), buf, cc->response_len);
 
-	free(buf);
+	xfree(buf);
 	return 0;
 } /* -- end set_response() -- */
 
@@ -444,29 +436,33 @@ get_sock_owner(uint lport, ulong rhost, uint rport, uid_t* owner) {
 		choke("Could not open /proc/net/tcp");
 		return ET_UNKNOWN;
 	}
-	if( getline(&line, &len, FP) < 0 ) {
+	int rc = getline(&line, &len, FP);
+	xmalloc_ptr_ref(line);
+	if( rc < 0 ) {
 		choke("Could not read /proc/net/tcp");
 		fclose(FP);
-		if_notnull_free(line);
+		xnullfree(line);
 		return ET_UNKNOWN;
 	}
 	/* discard the header */
-	if_notnull_free(line);
+	xnullfree(line);
+	len = 0;
 
 	struct procnettcp_info tcpi;
 	while(getline(&line, &len, FP) > 0) {
+		xmalloc_ptr_ref(line);
 		chomp(line);
 		if( is_wspace_only(line) ) {
-			if_notnull_free(line);
+			xnullfree(line);
 			continue;
 		}
 		if( get_sockinfo(&tcpi, line) ) {
 			carp("get_sockinfo() failed");
-			if_notnull_free(line);
+			xfree(line);
 			fclose(FP);
 			return ET_UNKNOWN;
 		}
-		if_notnull_free(line);
+		xnullfree(line);
 
 		if( tcpi.state != 1 )
 			continue;
@@ -503,28 +499,37 @@ prepare_for_write(ConnContext* cc) {
 		return -1;
 
 	uid_t owner;
-	char* answer;
+	char* answer = NULL;
 	c_error_t reterr = get_sock_owner(cc->lport, cc->rhost, cc->rport, &owner);
 	/* If error, return the appropriate string */
 	if( reterr ) {
-		dreport("Getting default answer");
-		answer = get_default_answer_string(reterr);
-	} else
-		answer = get_answer_string(owner);
-
-	/* Note that this duplicates some of the functionality
-	of get_answer_string() -- but if the uid is unnamed we
-	need to inform the sysadmin of the fact; it may be an
-	indication of a security breach */
-	char* username;
-	if( get_usernam_r(&username, owner) )
-		carp("Could not retrieve username for uid %d", owner);
-	else if( ! username ) {
-		carp("Warning! Socket owner has no username!");
-		struct in_addr remote_host;
-		remote_host.s_addr = htonl(cc->rhost);
-		carp("uid: %d, local port: %d, remote-host: %s:%d",
-				owner, cc->lport, inet_ntoa(remote_host), cc->rport);
+		dreport("Getting default error");
+		gen_error_string(&answer, &default_ans_conf, reterr);
+	} else {
+		char* username;
+		if( get_usernam_r(&username, owner) )
+			carp("Could not retrieve username for uid %d", owner);
+		else if( ! username ) {
+			carp("Warning! Socket owner has no username!");
+			struct in_addr remote_host;
+			remote_host.s_addr = htonl(cc->rhost);
+			carp("uid: %d, local port: %d, remote-host: %s:%d",
+					owner, cc->lport, inet_ntoa(remote_host), cc->rport);
+			AnswerConfig uuid_ac;
+			clear_answer_config(&uuid_ac);
+			cp_answer_config(&default_uuid_ans_conf, &uuid_ac);
+			uuid_ac.uid = owner;
+			gen_answer_string(&answer, &uuid_ac);
+		} else {
+			/* get cached AnswerConfig for user */
+			AnswerConfig* ac = NULL;
+			int reterr = get_cached_answer(&ac, owner, username);
+			if( reterr )
+				gen_error_string(&answer, &default_ans_conf, reterr);
+			else
+				answer = xstrdup(ac->cached_answer);
+		}
+		xnullfree(username);
 	}
 
 	if( ! answer )
@@ -602,11 +607,11 @@ we return with 0 and errno == EINTR */
 int
 sockloop_perform( void ) {
 	if( cc_list ) {
-		/* This is so ugly -- we have to iterate thru cc_list three times! */
+		/* This is so ugly -- we have to iterate thru cc_list twice! */
 		ConnContext* cc;
 
 		dreport("Checking reads");
-		/* And now we check for reads */
+		/* Check for reads */
 		cc = cc_list;
 		while( cc ) {
 			if( (cc->qsock >= 0) && FD_ISSET(cc->qsock, &rfds) ) {
@@ -618,7 +623,7 @@ sockloop_perform( void ) {
 		}
 
 		dreport("Checking writes");
-		/* next, we check writes */
+		/* Check for writes */
 		cc = cc_list;
 		while( cc ) {
 			if( (cc->qsock >= 0) && FD_ISSET(cc->qsock, &wfds) ) {
@@ -646,7 +651,7 @@ sockloop_perform( void ) {
 			return -1;
 		}
 		/* Place all OOB data into the normal stream */
-		int yes = TRUE;
+		int yes = true;
 		if( setsockopt(newcon, SOL_TCP, SO_OOBINLINE, &yes, sizeof yes) ) {
 			choke("Could not set OOBINLINE socket option");
 			exit(0);
@@ -672,10 +677,10 @@ sockloop_iter( void ) {
 		clean_cache();
 		return 0;
 	}
-	if( retval < 0 )
-		return -1;
 	if( errno == EINTR )
 		return 0;
+	if( retval < 0 )
+		return -1;
 
 	if( sockloop_perform() < 0 )
 		return -1;
@@ -687,7 +692,7 @@ sockloop_iter( void ) {
 0 on success, -1 on error, or if an unhandled signal was caught. */
 int
 sockloop_do( void ) {
-	while(TRUE) {
+	while(true) {
 		if( sockloop_iter() )
 			return -1;
 		/* So far all of our signals are handled;
